unit Skelet;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics;

type
  TSkelet=class
  private
    FParent: TSkelet;
    FdX, FdY: Integer;
    FX, FY: Integer;
    FChilds: TList;
    function GetChild(Index: Integer): TSkelet;
    function GetChildCount: Integer;
  protected
    procedure DoStep; virtual;
    procedure DoPaint(Canvas: TCanvas); virtual;
  public
    constructor Create(Parent: TSkelet; dX, dY: Integer);
    destructor Destroy; override;
    procedure Step;
    procedure Paint(Canvas: TCanvas);
    property Parent: TSkelet read FParent;
    property dX: Integer read FdX write FdX;
    property dY: Integer read FdY write FdY;
    property X: Integer read FX write FX;
    property Y: Integer read FY write FY;
    property Childs[Index: Integer]: TSkelet read GetChild;
    property ChildCount: Integer read GetChildCount;
  end;

implementation

// TSkelet - public:

constructor TSkelet.Create(Parent: TSkelet; dX, dY: Integer);
begin
  inherited Create;
  FParent:=Parent;
  FdX:=dX;
  FdY:=dY;
  FChilds:=TList.Create;
  if FParent<>nil then FParent.FChilds.Add(Self);
end;

destructor TSkelet.Destroy;
begin
  if FParent<>nil then FParent.FChilds.Remove(Self);
  if FChilds<>nil then begin
    while FChilds.Count>0 do
      TSkelet(FChilds[FChilds.Count-1]).Free;
    FChilds.Free;
  end;
  inherited;
end;

procedure TSkelet.Step;
var
  Index: Integer;
begin
  DoStep;
  for Index:=0 to FChilds.Count-1 do
    TSkelet(FChilds[Index]).Step;
end;

procedure TSkelet.Paint(Canvas: TCanvas);
var
  Index: Integer;
begin
  DoPaint(Canvas);
  for Index:=0 to FChilds.Count-1 do
    TSkelet(FChilds[Index]).Paint(Canvas);
end;



// TSkelet - protected:

procedure TSkelet.DoStep;
begin
  if Parent=nil then begin
    X:=dX;
    Y:=dY;
  end else begin
    X:=Parent.X+dX;
    Y:=Parent.Y+dY;
  end;
end;

procedure TSkelet.DoPaint(Canvas: TCanvas);
var
  XX, YY: Integer;
begin
  if Parent=nil then begin
    XX:=Round(X);
    YY:=Round(Y);
    Canvas.Ellipse(XX-4, YY-4, XX+4, YY+4);
  end else begin
    XX:=Round(Parent.X);
    YY:=Round(Parent.Y);
    Canvas.MoveTo(XX, YY);
    XX:=Round(X);
    YY:=Round(Y);
    Canvas.LineTo(XX, YY);
  end;
end;

// TSkelet - private:

function TSkelet.GetChild(Index: Integer): TSkelet;
begin
  Result:=FChilds[Index];
end;

function TSkelet.GetChildCount: Integer;
begin
  Result:=FChilds.Count;
end;

end.
